@@ -402,6 +402,455 @@ enum NavStatusFlags2 {
     Inactive = 3,
 }
 
+#[repr(transparent)]
+#[derive(Copy, Clone)]
+pub struct NavSatSvFlags(u32);
+
+impl NavSatSvFlags {
+    pub fn quality_ind(self) -> NavSatQualityIndicator {
+        let bits = self.0 & 0x7;
+        match bits {
+            0 => NavSatQualityIndicator::NoSignal,
+            1 => NavSatQualityIndicator::Searching,
+            2 => NavSatQualityIndicator::SignalAcquired,
+            3 => NavSatQualityIndicator::SignalDetected,
+            4 => NavSatQualityIndicator::CodeLock,
+            5 | 6 | 7 => NavSatQualityIndicator::CarrierLock,
+            _ => {
+                panic!("Unexpected 3-bit bitfield value {}!", bits);
+            }
+        }
+    }
+
+    pub fn sv_used(self) -> bool {
+        (self.0 >> 3) & 0x1 != 0
+    }
+
+    pub fn health(self) -> NavSatSvHealth {
+        let bits = (self.0 >> 4) & 0x3;
+        match bits {
+            1 => NavSatSvHealth::Healthy,
+            2 => NavSatSvHealth::Unhealthy,
+            x => NavSatSvHealth::Unknown(x as u8)
+        }
+    }
+
+    pub fn differential_correction_available(self) -> bool {
+        (self.0 >> 6) & 0x1 != 0
+    }
+
+    pub fn smoothed(self) -> bool {
+        (self.0 >> 7) & 0x1 != 0
+    }
+
+    pub fn orbit_source(self) -> NavSatOrbitSource {
+        let bits = (self.0 >> 8) & 0x7;
+        match bits {
+            0 => NavSatOrbitSource::NoInfoAvailable,
+            1 => NavSatOrbitSource::Ephemeris,
+            2 => NavSatOrbitSource::Almanac,
+            3 => NavSatOrbitSource::AssistNowOffline,
+            4 => NavSatOrbitSource::AssistNowAutonomous,
+            x => NavSatOrbitSource::Other(x as u8),
+        }
+    }
+
+    pub fn ephemeris_available(self) -> bool {
+        (self.0 >> 11) & 0x1 != 0
+    }
+
+    pub fn almanac_available(self) -> bool {
+        (self.0 >> 12) & 0x1 != 0
+    }
+
+    pub fn an_offline_available(self) -> bool {
+        (self.0 >> 13) & 0x1 != 0
+    }
+
+    pub fn an_auto_available(self) -> bool {
+        (self.0 >> 14) & 0x1 != 0
+    }
+
+    pub fn sbas_corr(self) -> bool {
+        (self.0 >> 16) & 0x1 != 0
+    }
+
+    pub fn rtcm_corr(self) -> bool {
+        (self.0 >> 17) & 0x1 != 0
+    }
+
+    pub fn slas_corr(self) -> bool {
+        (self.0 >> 18) & 0x1 != 0
+    }
+
+    pub fn spartn_corr(self) -> bool {
+        (self.0 >> 19) & 0x1 != 0
+    }
+
+    pub fn pr_corr(self) -> bool {
+        (self.0 >> 20) & 0x1 != 0
+    }
+
+    pub fn cr_corr(self) -> bool {
+        (self.0 >> 21) & 0x1 != 0
+    }
+
+    pub fn do_corr(self) -> bool {
+        (self.0 >> 22) & 0x1 != 0
+    }
+
+    pub const fn from(x: u32) -> Self {
+        Self(x)
+    }
+}
+
+impl fmt::Debug for NavSatSvFlags {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("NavSatSvFlags")
+            .field("quality_ind", &self.quality_ind())
+            .field("sv_used", &self.sv_used())
+            .field("health", &self.health())
+            .field("differential_correction_available", &self.differential_correction_available())
+            .field("smoothed", &self.smoothed())
+            .field("orbit_source", &self.orbit_source())
+            .field("ephemeris_available", &self.ephemeris_available())
+            .field("almanac_available", &self.almanac_available())
+            .field("an_offline_available", &self.an_offline_available())
+            .field("an_auto_available", &self.an_auto_available())
+            .field("sbas_corr", &self.sbas_corr())
+            .field("rtcm_corr", &self.rtcm_corr())
+            .field("slas_corr", &self.slas_corr())
+            .field("spartn_corr", &self.spartn_corr())
+            .field("pr_corr", &self.pr_corr())
+            .field("cr_corr", &self.cr_corr())
+            .field("do_corr", &self.do_corr())
+            .finish()
+    }
+}
+
+#[derive(Copy, Clone, Debug)]
+pub enum NavSatQualityIndicator {
+    NoSignal,
+    Searching,
+    SignalAcquired,
+    SignalDetected,
+    CodeLock,
+    CarrierLock,
+}
+
+#[derive(Copy, Clone, Debug)]
+pub enum NavSatSvHealth {
+    Healthy,
+    Unhealthy,
+    Unknown(u8),
+}
+
+#[derive(Copy, Clone, Debug)]
+pub enum NavSatOrbitSource {
+    NoInfoAvailable,
+    Ephemeris,
+    Almanac,
+    AssistNowOffline,
+    AssistNowAutonomous,
+    Other(u8),
+}
+
+#[ubx_packet_recv]
+#[ubx(class = 0x01, id = 0x35, fixed_payload_len = 12)]
+struct NavSatSvInfo {
+    gnss_id: u8,
+    sv_id: u8,
+    cno: u8,
+    elev: i8,
+    azim: i16,
+    pr_res: i16,
+
+    #[ubx(map_type = NavSatSvFlags)]
+    flags: u32,
+}
+
+/*impl fmt::Debug for NavSatSvInfoRef<'_> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("NavSatSvInfo")
+            .field("gnss_id", &self.gnss_id())
+            .field("sv_id", &self.sv_id())
+            .field("cno", &self.cno())
+            .field("elev", &self.elev())
+            .field("azim", &self.azim())
+            .field("pr_res", &self.pr_res())
+            .field("flags", &self.flags())
+            .finish()
+    }
+}*/
+
+pub struct NavSatIter<'a> {
+    data: &'a [u8],
+    offset: usize,
+}
+
+impl<'a> core::iter::Iterator for NavSatIter<'a> {
+    type Item = NavSatSvInfoRef<'a>;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.offset < self.data.len() {
+            let data = &self.data[self.offset..self.offset+12];
+            self.offset += 12;
+            Some(NavSatSvInfoRef(data))
+        } else {
+            None
+        }
+    }
+}
+
+impl fmt::Debug for NavSatIter<'_> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("NavSatIter")
+            .finish()
+    }
+}
+
+#[ubx_packet_recv]
+#[ubx(class = 0x01, id = 0x35, max_payload_len = 1240)]
+struct NavSat {
+    /// GPS time of week in ms
+    itow: u32,
+
+    /// Message version, should be 1
+    version: u8,
+
+    num_svs: u8,
+
+    reserved: u16,
+
+    #[ubx(map_type = NavSatIter,
+        may_fail,
+        is_valid = navsat::is_valid,
+        from = navsat::convert_to_iter,
+        get_as_ref)]
+    svs: [u8; 0]
+}
+
+mod navsat {
+    use super::NavSatIter;
+
+    pub(crate) fn convert_to_iter(bytes: &[u8]) -> NavSatIter {
+        NavSatIter {
+            data: bytes,
+            offset: 0,
+        }
+    }
+
+    pub(crate) fn is_valid(bytes: &[u8]) -> bool {
+        bytes.len() % 12 == 0
+    }
+}
+
+/// Odometer solution
+#[ubx_packet_recv]
+#[ubx(class = 0x01, id = 0x09, fixed_payload_len = 20)]
+struct NavOdo {
+    version: u8,
+    reserved: [u8; 3],
+    i_tow: u32,
+    distance: u32,
+    total_distance: u32,
+    distance_std: u32,
+}
+
+/// Reset odometer
+#[ubx_packet_send]
+#[ubx(class = 0x01, id = 0x10, fixed_payload_len = 0)]
+struct NavResetOdo { }
+
+/// Configure odometer
+#[ubx_packet_recv_send]
+#[ubx(class = 0x06, id = 0x1E, fixed_payload_len = 20, flags = "default_for_builder")]
+struct CfgOdo {
+    version: u8,
+    reserved: [u8; 3],
+    /// Odometer COG filter flags. See [OdoCogFilterFlags] for details.
+    #[ubx(map_type = OdoCogFilterFlags)]
+    flags: u8,
+    #[ubx(map_type = OdoProfile, may_fail)]
+    odo_cfg: u8,
+    reserved2: [u8; 6],
+    cog_max_speed: u8,
+    cog_max_pos_acc: u8,
+    reserved3: [u8; 2],
+    vel_lp_gain: u8,
+    cog_lp_gain: u8,
+    reserved4: [u8; 2],
+}
+
+#[ubx_extend_bitflags]
+#[ubx(from, into_raw, rest_reserved)]
+bitflags! {
+    #[derive(Default)]
+    pub struct OdoCogFilterFlags: u8 {
+        /// Odometer enabled flag
+        const USE_ODO = 0x01;
+        /// Low-speed COG filter enabled flag
+        const USE_COG = 0x02;
+        /// Output low-pass filtered velocity flag
+        const OUT_LP_VEL = 0x04;
+        /// Output low-pass filtered heading (COG) flag
+        const OUT_LP_COG = 0x08;
+    }
+}
+
+/// Odometer configuration profile
+#[ubx_extend]
+#[ubx(from_unchecked, into_raw, rest_error)]
+#[repr(u8)]
+#[derive(Clone, Copy, Debug)]
+pub enum OdoProfile {
+    Running = 0,
+    Cycling = 1,
+    Swimming = 2,
+    Car = 3,
+    Custom = 4,
+}
+
+impl Default for OdoProfile {
+    fn default() -> Self {
+        Self::Running
+    }
+}
+
+/// Information message conifg
+#[ubx_packet_recv_send]
+#[ubx(
+    class = 0x06,
+    id = 0x2,
+    fixed_payload_len = 10,
+    flags = "default_for_builder"
+)]
+struct CfgInf {
+    protocol_id: u8,
+    reserved: [u8; 3],
+    #[ubx(map_type = CfgInfMask)]
+    inf_msg_mask_0: u8,
+    #[ubx(map_type = CfgInfMask)]
+    inf_msg_mask_1: u8,
+    #[ubx(map_type = CfgInfMask)]
+    inf_msg_mask_2: u8,
+    #[ubx(map_type = CfgInfMask)]
+    inf_msg_mask_3: u8,
+    #[ubx(map_type = CfgInfMask)]
+    inf_msg_mask_4: u8,
+    #[ubx(map_type = CfgInfMask)]
+    inf_msg_mask_5: u8,
+}
+
+#[ubx_extend_bitflags]
+#[ubx(from, into_raw, rest_reserved)]
+bitflags! {
+    /// `CfgInfMask` parameters bitmask
+    #[derive(Default)]
+    pub struct CfgInfMask: u8 {
+        const ERROR = 0x1;
+        const WARNING = 0x2;
+        const NOTICE = 0x4;
+        const DEBUG = 0x08;
+        const TEST  = 0x10;
+    }
+}
+
+#[ubx_packet_recv]
+#[ubx(
+    class = 0x4,
+    id = 0x0,
+    max_payload_len = 1240,
+    flags = "default_for_builder"
+)]
+struct InfError{
+    #[ubx(map_type = Option<&str>, 
+        may_fail, 
+        is_valid = inf::is_valid,
+        from = inf::convert_to_str,
+        get_as_ref)]
+    message: [u8; 0]
+}
+
+#[ubx_packet_recv]
+#[ubx(
+    class = 0x4,
+    id = 0x2,
+    max_payload_len = 1240,
+    flags = "default_for_builder"
+)]
+struct InfNotice{
+    #[ubx(map_type = Option<&str>, 
+        may_fail, 
+        is_valid = inf::is_valid,
+        from = inf::convert_to_str,
+        get_as_ref)]
+    message: [u8; 0]
+}
+
+#[ubx_packet_recv]
+#[ubx(
+    class = 0x4,
+    id = 0x3,
+    max_payload_len = 1240,
+    flags = "default_for_builder"
+)]
+struct InfTest{
+    #[ubx(map_type = Option<&str>, 
+        may_fail, 
+        is_valid = inf::is_valid,
+        from = inf::convert_to_str,
+        get_as_ref)]
+    message: [u8; 0]
+}
+
+#[ubx_packet_recv]
+#[ubx(
+    class = 0x4,
+    id = 0x1,
+    max_payload_len = 1240,
+    flags = "default_for_builder"
+)]
+struct InfWarning{
+    #[ubx(map_type = Option<&str>, 
+        may_fail, 
+        is_valid = inf::is_valid,
+        from = inf::convert_to_str,
+        get_as_ref)]
+    message: [u8; 0]
+}
+
+#[ubx_packet_recv]
+#[ubx(
+    class = 0x4,
+    id = 0x4,
+    max_payload_len = 1240,
+    flags = "default_for_builder"
+)]
+struct InfDebug{
+    #[ubx(map_type = Option<&str>, 
+        may_fail, 
+        is_valid = inf::is_valid,
+        from = inf::convert_to_str,
+        get_as_ref)]
+    message: [u8; 0]
+}
+
+
+mod inf {
+	pub(crate) fn convert_to_str(bytes: &[u8]) -> Option<&str> {
+        match core::str::from_utf8(bytes) {
+            Ok(msg) => Some(msg),
+            Err(_) => None,
+        }
+    }
+
+    pub(crate) fn is_valid(_bytes: &[u8]) -> bool {
+        // Validity is checked in convert_to_str
+        true
+    }
+}
+
 #[ubx_packet_send]
 #[ubx(
     class = 0x0B,
@@ -513,6 +962,35 @@ struct CfgRst {
     reserved1: u8,
 }
 
+/// Reset Receiver / Clear Backup Data Structures
+#[ubx_packet_recv_send]
+#[ubx(class = 6, id = 0x13, fixed_payload_len = 4)]
+struct CfgAnt {
+    /// Antenna flag mask. See [AntFlags] for details.
+    #[ubx(map_type = AntFlags)]
+    flags: u16,
+    /// Antenna pin configuration. See 32.10.1.1 in receiver spec for details.
+    pins: u16,
+}
+
+#[ubx_extend_bitflags]
+#[ubx(from, into_raw, rest_reserved)]
+bitflags! {
+    #[derive(Default)]
+    pub struct AntFlags: u16 {
+        /// Enable supply voltage control signal
+        const SVCS = 0x01;
+        /// Enable short circuit detection
+        const SCD = 0x02;
+        /// Enable open circuit detection
+        const OCD = 0x04;
+        /// Power down on short circuit detection
+        const PDWN_ON_SCD = 0x08;
+        /// Enable automatic recovery from short circuit state
+        const RECOVERY = 0x10;
+    }
+}
+
 #[ubx_extend_bitflags]
 #[ubx(into_raw, rest_reserved)]
 bitflags! {
@@ -1020,6 +1498,225 @@ impl<T: FloatCore + FromPrimitive + ToPrimitive> ScaleBack<T> {
     }
 }
 
+/// Navigation Engine Expert Settings
+#[ubx_packet_recv_send]
+#[ubx(
+    class = 0x06,
+    id = 0x23,
+    fixed_payload_len = 40,
+    flags = "default_for_builder"
+)]
+struct CfgNavX5 {
+    /// Only version 2 supported
+    version: u16,
+
+    /// Only the masked parameters will be applied
+    #[ubx(map_type = CfgNavX5Params1)]
+    mask1: u16,
+
+    #[ubx(map_type = CfgNavX5Params2)]
+    mask2: u32,
+
+    /// Reserved
+    reserved1: [u8; 2],
+
+    /// Minimum number of satellites for navigation
+    min_svs: u8,
+
+    ///Maximum number of satellites for navigation
+    max_svs: u8,
+
+    /// Minimum satellite signal level for navigation
+    min_cno_dbhz: u8,
+
+    /// Reserved
+    reserved2: u8,
+
+    /// initial fix must be 3D
+    ini_fix_3d: u8,
+
+    /// Reserved
+    reserved3: [u8; 2],
+
+    /// issue acknowledgements for assistance message input
+    ack_aiding: u8,
+
+    /// GPS week rollover number
+    wkn_rollover: u16,
+
+    /// Permanently attenuated signal compensation
+    sig_atten_comp_mode: u8,
+
+    /// Reserved
+    reserved4: u8,
+    reserved5: [u8; 2],
+    reserved6: [u8; 2],
+
+    /// Use Precise Point Positioning (only available with the PPP product variant)
+    use_ppp: u8,
+
+    /// AssistNow Autonomous configuration
+    aop_cfg: u8,
+
+    /// Reserved
+    reserved7: [u8; 2],
+
+    /// Maximum acceptable (modeled) AssistNow Autonomous orbit error
+    aop_orb_max_err: u16,
+
+    /// Reserved
+    reserved8: [u8; 4],
+    reserved9: [u8; 3],
+
+    /// Enable/disable ADR/UDR sensor fusion 
+    use_adr: u8,
+}
+
+#[ubx_extend_bitflags]
+#[ubx(from, into_raw, rest_reserved)]
+bitflags! {
+    /// `CfgNavX51` parameters bitmask
+    #[derive(Default)]
+    pub struct CfgNavX5Params1: u16 {
+        /// apply min/max SVs settings
+        const MIN_MAX = 0x4;
+        /// apply minimum C/N0 setting
+        const MIN_CNO = 0x8;
+        /// apply initial 3D fix settings
+        const INITIAL_3D_FIX = 0x40;
+        /// apply GPS weeknumber rollover settings
+        const WKN_ROLL = 0x200;
+        /// apply assistance acknowledgement settings
+        const AID_ACK = 0x400;
+        /// apply usePPP flag
+        const USE_PPP = 0x2000;
+        /// apply aopCfg (useAOP flag) and aopOrbMaxErr settings (AssistNow Autonomous)
+        const AOP_CFG = 0x4000;
+    }
+}
+
+#[ubx_extend_bitflags]
+#[ubx(from, into_raw, rest_reserved)]
+bitflags! {
+    /// `CfgNavX5Params2` parameters bitmask
+    #[derive(Default)]
+    pub struct CfgNavX5Params2: u32 {
+        ///  apply ADR/UDR sensor fusion on/off setting
+        const USE_ADR = 0x40;
+        ///  apply signal attenuation compensation feature settings
+        const USE_SIG_ATTEN_COMP = 0x80;
+    }
+}
+
+/// GNSS Assistance ACK UBX-MGA-ACK
+#[ubx_packet_recv]
+#[ubx(class = 0x13, id = 0x60, fixed_payload_len = 8)]
+struct MgaAck {
+    /// Type of acknowledgment: 0 -> not used, 1 -> accepted
+    ack_type: u8,
+
+    /// Version 0
+    version: u8,
+
+    /// Provides greater information on what the receiver chose to do with the message contents.
+    /// See [MsgAckInfoCode].
+    #[ubx(map_type = MsgAckInfoCode)]
+    info_code: u8,
+
+    /// UBX message ID of the acknowledged message
+    msg_id: u8,
+
+    /// The first 4 bytes of the acknowledged message's payload
+    msg_payload_start: [u8; 4]
+}
+
+#[ubx_extend]
+#[ubx(from, rest_reserved)]
+#[repr(u8)]
+#[derive(Debug, Copy, Clone, PartialEq)]
+pub enum MsgAckInfoCode {
+    Accepted = 0,
+    RejectedNoTime = 1,
+    RejectedBadVersion = 2,
+    RejectedBadSize = 3,
+    RejectedDBStoreFailed= 4,
+    RejectedNotReady = 5,
+    RejectedUnknownType = 6,
+}
+
+/// Hardware status
+#[ubx_packet_recv]
+#[ubx(class = 0x0a, id = 0x09, fixed_payload_len = 60)]
+struct MonHw {
+    pin_sel: u32,
+    pin_bank: u32,
+    pin_dir: u32,
+    pin_val: u32,
+    noise_per_ms: u16,
+    agc_cnt: u16,
+    #[ubx(map_type = AntennaStatus)]
+    a_status: u8,
+    #[ubx(map_type = AntennaPower)]
+    a_power: u8,
+    flags: u8,
+    reserved1: u8,
+    used_mask: u32,
+    vp: [u8; 17],
+    jam_ind: u8,
+    reserved2: [u8; 2],
+    pin_irq: u32,
+    pull_h: u32,
+    pull_l: u32,
+}
+
+#[ubx_extend]
+#[ubx(from, rest_reserved)]
+#[repr(u8)]
+#[derive(Debug, Copy, Clone, PartialEq)]
+pub enum AntennaStatus {
+    Init = 0,
+    DontKnow = 1,
+    Ok = 2,
+    Short = 3,
+    Open = 4,
+}
+
+#[ubx_extend]
+#[ubx(from, rest_reserved)]
+#[repr(u8)]
+#[derive(Debug, Copy, Clone, PartialEq)]
+pub enum AntennaPower {
+    Off = 0,
+    On = 1,
+    DontKnow = 2,
+}
+
+pub struct MonVerExtensionIter<'a> {
+    data: &'a [u8],
+    offset: usize,
+}
+
+impl<'a> core::iter::Iterator for MonVerExtensionIter<'a> {
+    type Item = &'a str;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.offset < self.data.len() {
+            let data = &self.data[self.offset..self.offset+30];
+            self.offset += 30;
+            Some(mon_ver::convert_to_str_unchecked(data))
+        } else {
+            None
+        }
+    }
+}
+
+impl fmt::Debug for MonVerExtensionIter<'_> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("MonVerExtensionIter")
+            .finish()
+    }
+}
+
 /// Receiver/Software Version
 #[ubx_packet_recv]
 #[ubx(class = 0x0a, id = 0x04, max_payload_len = 1240)]
@@ -1032,13 +1729,15 @@ struct MonVer {
     hardware_version: [u8; 10],
 
     /// Extended software information strings
-    #[ubx(map_type = impl Iterator<Item=&str>, may_fail,
+    #[ubx(map_type = MonVerExtensionIter, may_fail,
           from = mon_ver::extension_to_iter,
           is_valid = mon_ver::is_extension_valid)]
     extension: [u8; 0],
 }
 
 mod mon_ver {
+    use super::MonVerExtensionIter;
+
     pub(crate) fn convert_to_str_unchecked(bytes: &[u8]) -> &str {
         let null_pos = bytes
             .iter()
@@ -1071,8 +1770,11 @@ mod mon_ver {
         }
     }
 
-    pub(crate) fn extension_to_iter(payload: &[u8]) -> impl Iterator<Item = &str> {
-        payload.chunks(30).map(|x| convert_to_str_unchecked(x))
+    pub(crate) fn extension_to_iter(payload: &[u8]) -> MonVerExtensionIter {
+        MonVerExtensionIter {
+            data: payload,
+            offset: 0,
+        }
     }
 }
 
@@ -1086,12 +1788,23 @@ define_recv_packets!(
         NavSolution,
         NavVelNed,
         NavTimeUTC,
+        NavSat,
+        NavOdo,
+        CfgOdo,
+        MgaAck,
         AlpSrv,
         AckAck,
         AckNak,
         CfgPrtSpi,
         CfgPrtUart,
         CfgNav5,
-        MonVer
+        CfgAnt,
+        InfError,
+        InfWarning,
+        InfNotice,
+        InfTest,
+        InfDebug,
+        MonVer,
+        MonHw
     }
 );
